@@ -87,7 +87,7 @@ PropertiesWidget::PropertiesWidget(QWidget *parent, MainWindow* main_window, Tra
   connect(selectAllButton, SIGNAL(clicked()), PropListModel, SLOT(selectAll()));
   connect(selectNoneButton, SIGNAL(clicked()), PropListModel, SLOT(selectNone()));
   connect(filesList, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayFilesListMenu(const QPoint&)));
-  connect(filesList, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(openDoubleClickedFile(QModelIndex)));
+  connect(filesList, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(openDoubleClickedFile(const QModelIndex &)));
   connect(PropListModel, SIGNAL(filteredFilesChanged()), this, SLOT(filteredFilesChanged()));
   connect(listWebSeeds, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayWebSeedListMenu(const QPoint&)));
   connect(transferList, SIGNAL(currentTorrentChanged(QTorrentHandle)), this, SLOT(loadTorrentInfos(QTorrentHandle)));
@@ -333,14 +333,14 @@ void PropertiesWidget::loadDynamicData() {
       // Update ratio info
       const qreal ratio = QBtSession::instance()->getRealRatio(h.hash());
       shareRatio->setText(ratio > QBtSession::MAX_RATIO ? QString::fromUtf8("∞") : misc::accurateDoubleToString(ratio, 2));
-      if (!h.is_seed()) {
+      if (!h.is_seed() && h.has_metadata()) {
         showPiecesDownloaded(true);
         // Downloaded pieces
         bitfield bf(h.get_torrent_info().num_pieces(), 0);
         h.downloading_pieces(bf);
         downloaded_pieces->setProgress(h.pieces(), bf);
         // Pieces availability
-        if (h.has_metadata() && !h.is_paused() && !h.is_queued() && !h.is_checking()) {
+        if (!h.is_paused() && !h.is_queued() && !h.is_checking()) {
           showPiecesAvailability(true);
           std::vector<int> avail;
           h.piece_availability(avail);
@@ -399,50 +399,68 @@ void PropertiesWidget::loadUrlSeeds() {
   }
 }
 
-void PropertiesWidget::openDoubleClickedFile(QModelIndex index) {
+void PropertiesWidget::openDoubleClickedFile(const QModelIndex &index) {
   if (!index.isValid()) return;
   if (!h.is_valid() || !h.has_metadata()) return;
-  if (PropListModel->itemType(index) == TorrentContentModelItem::FileType) {
-    int i = PropListModel->getFileIndex(index);
-    const QDir saveDir(h.save_path());
-    const QString filename = h.filepath_at(i);
-    const QString file_path = QDir::cleanPath(saveDir.absoluteFilePath(filename));
-    qDebug("Trying to open file at %s", qPrintable(file_path));
-    // Flush data
-    h.flush_cache();
-    if (QFile::exists(file_path)) {
-      QDesktopServices::openUrl(QUrl::fromLocalFile(file_path));
-    } else {
-      QMessageBox::warning(this, tr("I/O Error"), tr("This file does not exist yet."));
-    }
+  if (PropListModel->itemType(index) == TorrentContentModelItem::FileType)
+    openFile(index);
+  else
+    openFolder(index, false);
+}
+
+void PropertiesWidget::openFile(const QModelIndex &index) {
+  int i = PropListModel->getFileIndex(index);
+  const QDir saveDir(h.save_path());
+  const QString filename = h.filepath_at(i);
+  const QString file_path = QDir::cleanPath(saveDir.absoluteFilePath(filename));
+  qDebug("Trying to open file at %s", qPrintable(file_path));
+  // Flush data
+  h.flush_cache();
+  if (QFile::exists(file_path)) {
+    QDesktopServices::openUrl(QUrl::fromLocalFile(file_path));
+  }
+  else {
+    QMessageBox::warning(this, tr("I/O Error"), tr("This file does not exist yet."));
+  }
+}
+
+void PropertiesWidget::openFolder(const QModelIndex &index, bool containing_folder) {
+  // FOLDER
+  QStringList path_items;
+  path_items << index.data().toString();
+  QModelIndex parent = PropListModel->parent(index);
+  while(parent.isValid()) {
+    path_items.prepend(parent.data().toString());
+    parent = PropListModel->parent(parent);
+  }
+  if (path_items.isEmpty())
+    return;
+  if (containing_folder)
+    path_items.removeLast();
+  const QDir saveDir(h.save_path());
+  const QString filename = path_items.join(QDir::separator());
+  const QString file_path = QDir::cleanPath(saveDir.absoluteFilePath(filename));
+  qDebug("Trying to open folder at %s", qPrintable(file_path));
+  // Flush data
+  h.flush_cache();
+  if (QFile::exists(file_path)) {
+    QDesktopServices::openUrl(QUrl::fromLocalFile(file_path));
   } else {
-    // FOLDER
-    QStringList path_items;
-    path_items << index.data().toString();
-    QModelIndex parent = PropListModel->parent(index);
-    while(parent.isValid()) {
-      path_items.prepend(parent.data().toString());
-      parent = PropListModel->parent(parent);
-    }
-    const QDir saveDir(h.save_path());
-    const QString filename = path_items.join(QDir::separator());
-    const QString file_path = QDir::cleanPath(saveDir.absoluteFilePath(filename));
-    qDebug("Trying to open folder at %s", qPrintable(file_path));
-    // Flush data
-    h.flush_cache();
-    if (QFile::exists(file_path)) {
-      QDesktopServices::openUrl(QUrl::fromLocalFile(file_path));
-    } else {
-      QMessageBox::warning(this, tr("I/O Error"), tr("This folder does not exist yet."));
-    }
+    QMessageBox::warning(this, tr("I/O Error"), tr("This folder does not exist yet."));
   }
 }
 
 void PropertiesWidget::displayFilesListMenu(const QPoint&) {
+  if (!h.is_valid())
+    return;
   QMenu myFilesLlistMenu;
   QModelIndexList selectedRows = filesList->selectionModel()->selectedRows(0);
+  QAction *actOpen = 0;
+  QAction *actOpenContainingFolder = 0;
   QAction *actRename = 0;
   if (selectedRows.size() == 1) {
+    actOpen = myFilesLlistMenu.addAction(tr("Open"));
+    actOpenContainingFolder = myFilesLlistMenu.addAction(IconProvider::instance()->getIcon("inode-directory"), tr("Open Containing Folder"));
     actRename = myFilesLlistMenu.addAction(IconProvider::instance()->getIcon("edit-rename"), tr("Rename..."));
     myFilesLlistMenu.addSeparator();
   }
@@ -461,22 +479,27 @@ void PropertiesWidget::displayFilesListMenu(const QPoint&) {
   }
   // Call menu
   const QAction *act = myFilesLlistMenu.exec(QCursor::pos());
+  // The selected torrent might have dissapeared during exec()
+  // from the current view thus leaving invalid indices.
+  const QModelIndex index = *(selectedRows.begin());
+  if (!index.isValid())
+    return;
   if (act) {
-    if (act == actRename) {
+    if (act == actOpen)
+      openDoubleClickedFile(index);
+    else if (act == actOpenContainingFolder)
+      openFolder(index, true);
+    else if (act == actRename)
       renameSelectedFile();
-    } else {
-      int prio = 1;
-      if (act == actionHigh) {
+    else {
+      int prio = prio::NORMAL;
+      if (act == actionHigh)
         prio = prio::HIGH;
-      } else {
-        if (act == actionMaximum) {
-          prio = prio::MAXIMUM;
-        } else {
-          if (act == actionNot_downloaded) {
-            prio = prio::IGNORED;
-          }
-        }
-      }
+      else if (act == actionMaximum)
+        prio = prio::MAXIMUM;
+      else if (act == actionNot_downloaded)
+        prio = prio::IGNORED;
+
       qDebug("Setting files priority");
       foreach (QModelIndex index, selectedRows) {
         qDebug("Setting priority(%d) for file at row %d", prio, index.row());
@@ -489,6 +512,8 @@ void PropertiesWidget::displayFilesListMenu(const QPoint&) {
 }
 
 void PropertiesWidget::displayWebSeedListMenu(const QPoint&) {
+  if (!h.is_valid())
+    return;
   QMenu seedMenu;
   QModelIndexList rows = listWebSeeds->selectionModel()->selectedRows();
   QAction *actAdd = seedMenu.addAction(IconProvider::instance()->getIcon("list-add"), tr("New Web seed"));
@@ -521,6 +546,8 @@ void PropertiesWidget::renameSelectedFile() {
   if (selectedIndexes.size() != 1)
     return;
   const QModelIndex index = selectedIndexes.first();
+  if (!index.isValid())
+    return;
   // Ask for new name
   bool ok;
   QString new_name_last = AutoExpandableDialog::getText(this, tr("Rename the file"),
