@@ -29,11 +29,10 @@
  */
 
 #include "btjson.h"
-#include "jsondict.h"
-#include "jsonlist.h"
 #include "misc.h"
 #include "qbtsession.h"
 #include "torrentpersistentdata.h"
+#include "jsonutils.h"
 
 #if QT_VERSION >= QT_VERSION_CHECK(4, 7, 0)
 #include <QElapsedTimer>
@@ -48,20 +47,20 @@ using namespace libtorrent;
   static QElapsedTimer cacheTimer; \
   static bool initialized = false; \
   if (initialized && !cacheTimer.hasExpired(DUR)) \
-    return VAR.toString(); \
+    return json::toJson(VAR); \
   initialized = true; \
   cacheTimer.start(); \
-  VAR.clear()
+  VAR = VARTYPE()
 
 #define CACHED_VARIABLE_FOR_HASH(VARTYPE, VAR, DUR, HASH) \
   static VARTYPE VAR; \
   static QString prev_hash; \
   static QElapsedTimer cacheTimer; \
   if (prev_hash == HASH && !cacheTimer.hasExpired(DUR)) \
-    return VAR.toString(); \
+    return json::toJson(VAR); \
   prev_hash = HASH; \
   cacheTimer.start(); \
-  VAR.clear()
+  VAR = VARTYPE()
 
 #else
 // We don't support caching for Qt < 4.7 at the moment
@@ -121,29 +120,35 @@ static const char KEY_FILE_IS_SEED[] = "is_seed";
 static const char KEY_TRANSFER_DLSPEED[] = "dl_info";
 static const char KEY_TRANSFER_UPSPEED[] = "up_info";
 
-static JsonDict toJson(const QTorrentHandle& h)
+static QVariantMap toMap(const QTorrentHandle& h)
 {
-  JsonDict ret;
-  ret.add(KEY_TORRENT_HASH, h.hash());
-  ret.add(KEY_TORRENT_NAME, h.name());
-  ret.add(KEY_TORRENT_SIZE, misc::friendlyUnit(h.actual_size())); // FIXME: Should pass as Number, not formatted String (for sorting).
-  ret.add(KEY_TORRENT_PROGRESS, (double)h.progress());
-  ret.add(KEY_TORRENT_DLSPEED, misc::friendlyUnit(h.download_payload_rate(), true)); // FIXME: Should be passed as a Number
-  ret.add(KEY_TORRENT_UPSPEED, misc::friendlyUnit(h.upload_payload_rate(), true)); // FIXME: Should be passed as a Number
+#if LIBTORRENT_VERSION_NUM >= 1600
+  libtorrent::torrent_status status = h.status(torrent_handle::query_accurate_download_counters);
+#else
+  libtorrent::torrent_status status = h.status();
+#endif
+
+  QVariantMap ret;
+  ret[KEY_TORRENT_HASH] =  h.hash();
+  ret[KEY_TORRENT_NAME] =  h.name();
+  ret[KEY_TORRENT_SIZE] =  misc::friendlyUnit(status.total_wanted, false, true); // FIXME: Should pass as Number, not formatted String (for sorting).
+  ret[KEY_TORRENT_PROGRESS] =  (double)h.progress();
+  ret[KEY_TORRENT_DLSPEED] = misc::friendlyUnit(status.download_payload_rate, true, true); // FIXME: Should be passed as a Number
+  ret[KEY_TORRENT_UPSPEED] = misc::friendlyUnit(status.upload_payload_rate, true, true); // FIXME: Should be passed as a Number
   if (QBtSession::instance()->isQueueingEnabled() && h.queue_position() >= 0)
-    ret.add(KEY_TORRENT_PRIORITY, QString::number(h.queue_position()));
+    ret[KEY_TORRENT_PRIORITY] =  QString::number(h.queue_position());
   else
-    ret.add(KEY_TORRENT_PRIORITY, "*");
-  QString seeds = QString::number(h.num_seeds());
-  if (h.num_complete() > 0)
-    seeds += " ("+QString::number(h.num_complete())+")";
-  ret.add(KEY_TORRENT_SEEDS, seeds);
-  QString leechs = QString::number(h.num_peers() - h.num_seeds());
-  if (h.num_incomplete() > 0)
-    leechs += " ("+QString::number(h.num_incomplete())+")";
-  ret.add(KEY_TORRENT_LEECHS, leechs);
+    ret[KEY_TORRENT_PRIORITY] =  "*";
+  QString seeds = QString::number(status.num_seeds);
+  if (status.num_complete > 0)
+    seeds += " ("+QString::number(status.num_complete)+")";
+  ret[KEY_TORRENT_SEEDS] =  seeds;
+  QString leechs = QString::number(status.num_peers - status.num_seeds);
+  if (status.num_incomplete > 0)
+    leechs += " ("+QString::number(status.num_incomplete)+")";
+  ret[KEY_TORRENT_LEECHS] =  leechs;
   const qreal ratio = QBtSession::instance()->getRealRatio(h.hash());
-  ret.add(KEY_TORRENT_RATIO, (ratio > 100.) ? QString::fromUtf8("∞") : misc::accurateDoubleToString(ratio, 1));
+  ret[KEY_TORRENT_RATIO] = (ratio > 100.) ? QString::fromUtf8("∞") : misc::accurateDoubleToString(ratio, 1, false);
   QString eta;
   QString state;
   if (h.is_paused()) {
@@ -155,10 +160,10 @@ static JsonDict toJson(const QTorrentHandle& h)
     if (QBtSession::instance()->isQueueingEnabled() && h.is_queued())
       state = h.is_seed() ? "queuedUP" : "queuedDL";
     else {
-      switch (h.state()) {
+      switch (status.state) {
       case torrent_status::finished:
       case torrent_status::seeding:
-        state = h.upload_payload_rate() > 0 ? "uploading" : "stalledUP";
+        state = status.upload_payload_rate > 0 ? "uploading" : "stalledUP";
         break;
       case torrent_status::allocating:
       case torrent_status::checking_files:
@@ -168,7 +173,7 @@ static JsonDict toJson(const QTorrentHandle& h)
         break;
       case torrent_status::downloading:
       case torrent_status::downloading_metadata:
-        state = h.download_payload_rate() > 0 ? "downloading" : "stalledDL";
+        state = status.download_payload_rate > 0 ? "downloading" : "stalledDL";
         eta = misc::userFriendlyDuration(QBtSession::instance()->getETA(h.hash()));
         break;
       default:
@@ -176,8 +181,8 @@ static JsonDict toJson(const QTorrentHandle& h)
       }
     }
   }
-  ret.add(KEY_TORRENT_ETA, eta.isEmpty() ? QString::fromUtf8("∞") : eta);
-  ret.add(KEY_TORRENT_STATE, state);
+  ret[KEY_TORRENT_ETA] =  eta.isEmpty() ? QString::fromUtf8("∞") : eta;
+  ret[KEY_TORRENT_STATE] =  state;
 
   return ret;
 }
@@ -200,16 +205,16 @@ static JsonDict toJson(const QTorrentHandle& h)
  *   - "eta": Torrent ETA
  *   - "state": Torrent state
  */
-QString btjson::getTorrents()
+QByteArray btjson::getTorrents()
 {
-  CACHED_VARIABLE(JsonList, torrent_list, CACHE_DURATION_MS);
+  CACHED_VARIABLE(QVariantList, torrent_list, CACHE_DURATION_MS);
   std::vector<torrent_handle> torrents = QBtSession::instance()->getTorrents();
   std::vector<torrent_handle>::const_iterator it = torrents.begin();
   std::vector<torrent_handle>::const_iterator end = torrents.end();
   for( ; it != end; ++it) {
-    torrent_list.append(toJson(QTorrentHandle(*it)));
+    torrent_list.append(toMap(QTorrentHandle(*it)));
   }
-  return torrent_list.toString();
+  return json::toJson(torrent_list);
 }
 
 /**
@@ -222,9 +227,9 @@ QString btjson::getTorrents()
  *   - "num_peers": Tracker peer count
  *   - "msg": Tracker message (last)
  */
-QString btjson::getTrackersForTorrent(const QString& hash)
+QByteArray btjson::getTrackersForTorrent(const QString& hash)
 {
-  CACHED_VARIABLE_FOR_HASH(JsonList, tracker_list, CACHE_DURATION_MS, hash);
+  CACHED_VARIABLE_FOR_HASH(QVariantList, tracker_list, CACHE_DURATION_MS, hash);
   try {
     QTorrentHandle h = QBtSession::instance()->getTorrentHandle(hash);
     QHash<QString, TrackerInfos> trackers_data = QBtSession::instance()->getTrackersInfo(hash);
@@ -232,9 +237,9 @@ QString btjson::getTrackersForTorrent(const QString& hash)
     std::vector<announce_entry>::const_iterator it = vect_trackers.begin();
     std::vector<announce_entry>::const_iterator end = vect_trackers.end();
     for (; it != end; ++it) {
-      JsonDict tracker_dict;
+      QVariantMap tracker_dict;
       const QString tracker_url = misc::toQString(it->url);
-      tracker_dict.add(KEY_TRACKER_URL, tracker_url);
+      tracker_dict[KEY_TRACKER_URL] = tracker_url;
       const TrackerInfos data = trackers_data.value(tracker_url, TrackerInfos(tracker_url));
       QString status;
       if (it->verified)
@@ -245,18 +250,18 @@ QString btjson::getTrackersForTorrent(const QString& hash)
         else
           status = it->fails > 0 ? tr("Not working") : tr("Not contacted yet");
       }
-      tracker_dict.add(KEY_TRACKER_STATUS, status);
-      tracker_dict.add(KEY_TRACKER_PEERS, QString::number(trackers_data.value(tracker_url, TrackerInfos(tracker_url)).num_peers));
-      tracker_dict.add(KEY_TRACKER_MSG, data.last_message.trimmed());
+      tracker_dict[KEY_TRACKER_STATUS] = status;
+      tracker_dict[KEY_TRACKER_PEERS] = QString::number(trackers_data.value(tracker_url, TrackerInfos(tracker_url)).num_peers);
+      tracker_dict[KEY_TRACKER_MSG] = data.last_message.trimmed();
 
       tracker_list.append(tracker_dict);
     }
   } catch(const std::exception& e) {
     qWarning() << Q_FUNC_INFO << "Invalid torrent: " << e.what();
-    return QString();
+    return QByteArray();
   }
 
-  return tracker_list.toString();
+  return json::toJson(tracker_list);
 }
 
 /**
@@ -277,41 +282,47 @@ QString btjson::getTrackersForTorrent(const QString& hash)
  *   - "nb_connections": Torrent connection count
  *   - "share_ratio": Torrent share ratio
  */
-QString btjson::getPropertiesForTorrent(const QString& hash)
+QByteArray btjson::getPropertiesForTorrent(const QString& hash)
 {
-  CACHED_VARIABLE_FOR_HASH(JsonDict, data, CACHE_DURATION_MS, hash);
+  CACHED_VARIABLE_FOR_HASH(QVariantMap, data, CACHE_DURATION_MS, hash);
   try {
     QTorrentHandle h = QBtSession::instance()->getTorrentHandle(hash);
 
+#if LIBTORRENT_VERSION_NUM >= 1600
+    libtorrent::torrent_status status = h.status(torrent_handle::query_accurate_download_counters);
+    if (!status.has_metadata)
+#else
+    libtorrent::torrent_status status = h.status();
     if (!h.has_metadata())
-      return QString();
+#endif
+      return QByteArray();
 
     // Save path
     QString save_path = TorrentPersistentData::getSavePath(hash);
     if (save_path.isEmpty())
       save_path = h.save_path();
-    data.add(KEY_PROP_SAVE_PATH, save_path);
-    data.add(KEY_PROP_CREATION_DATE, h.creation_date());
-    data.add(KEY_PROP_PIECE_SIZE, misc::friendlyUnit(h.piece_length()));
-    data.add(KEY_PROP_COMMENT, h.comment());
-    data.add(KEY_PROP_WASTED, misc::friendlyUnit(h.total_failed_bytes() + h.total_redundant_bytes()));
-    data.add(KEY_PROP_UPLOADED, QString(misc::friendlyUnit(h.all_time_upload()) + " (" + misc::friendlyUnit(h.total_payload_upload()) + " " + tr("this session") + ")"));
-    data.add(KEY_PROP_DOWNLOADED, QString(misc::friendlyUnit(h.all_time_download()) + " (" + misc::friendlyUnit(h.total_payload_download()) + " " + tr("this session") + ")"));
-    data.add(KEY_PROP_UP_LIMIT, h.upload_limit() <= 0 ? QString::fromUtf8("∞") : misc::friendlyUnit(h.upload_limit(), true));
-    data.add(KEY_PROP_DL_LIMIT, h.download_limit() <= 0 ? QString::fromUtf8("∞") : misc::friendlyUnit(h.download_limit(), true));
+    data[KEY_PROP_SAVE_PATH] = save_path;
+    data[KEY_PROP_CREATION_DATE] = h.creation_date();
+    data[KEY_PROP_PIECE_SIZE] = misc::friendlyUnit(h.piece_length(), false, true);
+    data[KEY_PROP_COMMENT] = h.comment();
+    data[KEY_PROP_WASTED] = misc::friendlyUnit(status.total_failed_bytes + status.total_redundant_bytes, false, true);
+    data[KEY_PROP_UPLOADED] = QString(misc::friendlyUnit(status.all_time_upload, false, true) + " (" + misc::friendlyUnit(status.total_payload_upload, false, true) + " " + tr("this session") + ")");
+    data[KEY_PROP_DOWNLOADED] = QString(misc::friendlyUnit(status.all_time_download, false, true) + " (" + misc::friendlyUnit(status.total_payload_download, false, true) + " " + tr("this session") + ")");
+    data[KEY_PROP_UP_LIMIT] = h.upload_limit() <= 0 ? QString::fromUtf8("∞") : misc::friendlyUnit(h.upload_limit(), true, true);
+    data[KEY_PROP_DL_LIMIT] = h.download_limit() <= 0 ? QString::fromUtf8("∞") : misc::friendlyUnit(h.download_limit(), true, true);
     QString elapsed_txt = misc::userFriendlyDuration(h.active_time());
     if (h.is_seed())
       elapsed_txt += " ("+tr("Seeded for %1", "e.g. Seeded for 3m10s").arg(misc::userFriendlyDuration(h.seeding_time()))+")";
-    data.add(KEY_PROP_TIME_ELAPSED, elapsed_txt);
-    data.add(KEY_PROP_CONNECT_COUNT, QString(QString::number(h.num_connections()) + " (" + tr("%1 max", "e.g. 10 max").arg(QString::number(h.connections_limit())) + ")"));
+    data[KEY_PROP_TIME_ELAPSED] = elapsed_txt;
+    data[KEY_PROP_CONNECT_COUNT] = QString(QString::number(status.num_connections) + " (" + tr("%1 max", "e.g. 10 max").arg(QString::number(status.connections_limit)) + ")");
     const qreal ratio = QBtSession::instance()->getRealRatio(h.hash());
-    data.add(KEY_PROP_RATIO, ratio > 100. ? QString::fromUtf8("∞") : misc::accurateDoubleToString(ratio, 1));
+    data[KEY_PROP_RATIO] = ratio > 100. ? QString::fromUtf8("∞") : misc::accurateDoubleToString(ratio, 1, false);
   } catch(const std::exception& e) {
     qWarning() << Q_FUNC_INFO << "Invalid torrent: " << e.what();
-    return QString();
+    return QByteArray();
   }
 
-  return data.toString();
+  return json::toJson(data);
 }
 
 /**
@@ -325,38 +336,38 @@ QString btjson::getPropertiesForTorrent(const QString& hash)
  *   - "priority": File priority
  *   - "is_seed": Flag indicating if torrent is seeding/complete
  */
-QString btjson::getFilesForTorrent(const QString& hash)
+QByteArray btjson::getFilesForTorrent(const QString& hash)
 {
-  CACHED_VARIABLE_FOR_HASH(JsonList, file_list, CACHE_DURATION_MS, hash);
+  CACHED_VARIABLE_FOR_HASH(QVariantList, file_list, CACHE_DURATION_MS, hash);
   try {
     QTorrentHandle h = QBtSession::instance()->getTorrentHandle(hash);
     if (!h.has_metadata())
-      return QString();
+      return QByteArray();
 
     const std::vector<int> priorities = h.file_priorities();
     std::vector<size_type> fp;
     h.file_progress(fp);
     for (int i = 0; i < h.num_files(); ++i) {
-      JsonDict file_dict;
+      QVariantMap file_dict;
       QString fileName = h.filename_at(i);
       if (fileName.endsWith(".!qB", Qt::CaseInsensitive))
         fileName.chop(4);
-      file_dict.add(KEY_FILE_NAME, fileName);
+      file_dict[KEY_FILE_NAME] = fileName;
       const size_type size = h.filesize_at(i);
-      file_dict.add(KEY_FILE_SIZE, misc::friendlyUnit(size));
-      file_dict.add(KEY_FILE_PROGRESS, (size > 0) ? (fp[i] / (double) size) : 1.);
-      file_dict.add(KEY_FILE_PRIORITY, priorities[i]);
+      file_dict[KEY_FILE_SIZE] = misc::friendlyUnit(size, false, true);
+      file_dict[KEY_FILE_PROGRESS] = (size > 0) ? (fp[i] / (double) size) : 1.;
+      file_dict[KEY_FILE_PRIORITY] = priorities[i];
       if (i == 0)
-        file_dict.add(KEY_FILE_IS_SEED, h.is_seed());
+        file_dict[KEY_FILE_IS_SEED] = h.is_seed();
 
       file_list.append(file_dict);
     }
   } catch (const std::exception& e) {
     qWarning() << Q_FUNC_INFO << "Invalid torrent: " << e.what();
-    return QString();
+    return QByteArray();
   }
 
-  return file_list.toString();
+  return json::toJson(file_list);
 }
 
 /**
@@ -367,11 +378,11 @@ QString btjson::getFilesForTorrent(const QString& hash)
  *   - "dl_info": Global download info
  *   - "up_info": Global upload info
  */
-QString btjson::getTransferInfo()
+QByteArray btjson::getTransferInfo()
 {
-  CACHED_VARIABLE(JsonDict, info, CACHE_DURATION_MS);
+  CACHED_VARIABLE(QVariantMap, info, CACHE_DURATION_MS);
   session_status sessionStatus = QBtSession::instance()->getSessionStatus();
-  info.add(KEY_TRANSFER_DLSPEED, tr("D: %1/s - T: %2", "Download speed: x KiB/s - Transferred: x MiB").arg(misc::friendlyUnit(sessionStatus.payload_download_rate)).arg(misc::friendlyUnit(sessionStatus.total_payload_download)));
-  info.add(KEY_TRANSFER_UPSPEED, tr("U: %1/s - T: %2", "Upload speed: x KiB/s - Transferred: x MiB").arg(misc::friendlyUnit(sessionStatus.payload_upload_rate)).arg(misc::friendlyUnit(sessionStatus.total_payload_upload)));
-  return info.toString();
+  info[KEY_TRANSFER_DLSPEED] = tr("D: %1/s - T: %2", "Download speed: x KiB/s - Transferred: x MiB").arg(misc::friendlyUnit(sessionStatus.payload_download_rate, true, true)).arg(misc::friendlyUnit(sessionStatus.total_payload_download, false, true));
+  info[KEY_TRANSFER_UPSPEED] = tr("U: %1/s - T: %2", "Upload speed: x KiB/s - Transferred: x MiB").arg(misc::friendlyUnit(sessionStatus.payload_upload_rate, true, true)).arg(misc::friendlyUnit(sessionStatus.total_payload_upload, false, true));
+  return json::toJson(info);
 }
